@@ -15,71 +15,85 @@
 # limitations under the License.                                           #
 ############################################################################
 import json
+from concurrent.futures import Future
+from functools import partial
+from pathlib import Path
+from typing import Optional
 
 import pytest
-from pygls.protocol import (JsonRPCNotification, JsonRPCRequestMessage,
-                            JsonRPCResponseMessage, deserialize_message,
-                            to_lsp_name)
-from pygls.types import InitializeResult
+from pygls.exceptions import JsonRpcException, JsonRpcInvalidParams
+from pygls.lsp import Model, get_method_params_type
+from pygls.lsp.types import ClientCapabilities, InitializeParams, InitializeResult
+from pygls.protocol import JsonRPCNotification, JsonRPCRequestMessage, JsonRPCResponseMessage
+from pygls.protocol import deserialize_message as _deserialize_message
+from pygls.protocol import to_lsp_name
+
+TEST_METHOD = 'test_method'
 
 
-class dictToObj:
-    def __init__(self, entries):
-        self.__dict__.update(**entries)
+class FeatureParams(Model):
+    class InnerType(Model):
+        inner_field: str
+
+    field_a: str
+    field_b: Optional[InnerType] = None
 
 
-def test_deserialize_message_with_reserved_words_should_pass_without_errors(client_server):
+TEST_LSP_METHODS_MAP = {
+    TEST_METHOD: (None, FeatureParams, None),
+}
+
+deserialize_message = partial(
+    _deserialize_message,
+    get_params_type=partial(get_method_params_type, lsp_methods_map=TEST_LSP_METHODS_MAP)
+)
+
+
+def test_deserialize_notification_message_valid_params():
     params = '''
     {
         "jsonrpc": "2.0",
-        "method": "initialized",
+        "method": "test_method",
         "params": {
-            "__dummy__": true
+            "field_a": "test_a",
+            "field_b": {
+                "inner_field": "test_inner"
+            }
         }
     }
     '''
-    result = json.loads(params, object_hook=deserialize_message)
 
-    assert isinstance(result, JsonRPCNotification)
-    assert result.params._0 is True
-
-
-def test_deserialize_message_should_return_notification_message():
-    params = '''
-    {
-        "jsonrpc": "2.0",
-        "method": "test",
-        "params": "1"
-    }
-    '''
     result = json.loads(params, object_hook=deserialize_message)
 
     assert isinstance(result, JsonRPCNotification)
     assert result.jsonrpc == "2.0"
-    assert result.method == "test"
-    assert result.params == "1"
+
+    assert isinstance(result.params, FeatureParams)
+    assert result.params.field_a == "test_a"
+
+    assert isinstance(result.params.field_b, FeatureParams.InnerType)
+    assert result.params.field_b.inner_field == "test_inner"
 
 
-def test_deserialize_message_without_jsonrpc_field__should_return_object():
+def test_deserialize_notification_message_bad_params_should_raise_error():
     params = '''
     {
-        "random": "data",
-        "def": "def"
+        "jsonrpc": "2.0",
+        "method": "test_method",
+        "params": {
+            "field_a": "test_a",
+            "field_b": {
+                "wrong_field_name": "test_inner"
+            }
+        }
     }
     '''
-    result = json.loads(params, object_hook=deserialize_message)
 
-    assert type(result).__name__ == 'Object'
-    assert result.random == "data"
-
-    # namedtuple does not guarantee field order
-    try:
-        assert result._0 == "def"
-    except AttributeError:
-        assert result._1 == "def"
+    with pytest.raises(JsonRpcInvalidParams):
+        json.loads(params, object_hook=deserialize_message)
 
 
-def test_deserialize_message_should_return_response_message():
+def test_deserialize_response_message():
     params = '''
     {
         "jsonrpc": "2.0",
@@ -96,13 +110,18 @@ def test_deserialize_message_should_return_response_message():
     assert result.error is None
 
 
-def test_deserialize_message_should_return_request_message():
+def test_deserialize_request_message_with_registered_type__should_return_instance():
     params = '''
     {
         "jsonrpc": "2.0",
         "id": "id",
-        "method": "test",
-        "params": "1"
+        "method": "test_method",
+        "params": {
+            "field_a": "test_a",
+            "field_b": {
+                "inner_field": "test_inner"
+            }
+        }
     }
     '''
     result = json.loads(params, object_hook=deserialize_message)
@@ -110,8 +129,37 @@ def test_deserialize_message_should_return_request_message():
     assert isinstance(result, JsonRPCRequestMessage)
     assert result.jsonrpc == "2.0"
     assert result.id == "id"
-    assert result.method == "test"
-    assert result.params == "1"
+
+    assert isinstance(result.params, FeatureParams)
+    assert result.params.field_a == "test_a"
+
+    assert isinstance(result.params.field_b, FeatureParams.InnerType)
+    assert result.params.field_b.inner_field == "test_inner"
+
+
+def test_deserialize_request_message_without_registered_type__should_return_namedtuple():
+    params = '''
+    {
+        "jsonrpc": "2.0",
+        "id": "id",
+        "method": "random",
+        "params": {
+            "field_a": "test_a",
+            "field_b": {
+                "inner_field": "test_inner"
+            }
+        }
+    }
+    '''
+    result = json.loads(params, object_hook=deserialize_message)
+
+    assert isinstance(result, JsonRPCRequestMessage)
+    assert result.jsonrpc == "2.0"
+    assert result.id == "id"
+
+    assert type(result.params).__name__ == "Object"
+    assert result.params.field_a == "test_a"
+    assert result.params.field_b.inner_field == "test_inner"
 
 
 def test_data_received_without_content_type_should_handle_message(client_server):
@@ -183,67 +231,41 @@ def test_data_received_multi_message_should_handle_messages(client_server):
     server.lsp.data_received(data)
 
 
-def test_initialize_without_capabilities_should_raise_error(client_server):
+def test_data_received_error_should_raise_jsonrpc_error(client_server):
     _, server = client_server
-    params = dictToObj({
-        "processId": 1234,
-        "rootUri": None
+    body = json.dumps({
+        "jsonrpc": "2.0",
+        "id": "err",
+        "error": {
+            "code": -1,
+            "message": "message for you sir",
+        },
     })
-    with pytest.raises(Exception):
-        server.lsp.bf_initialize(params)
-
-
-def test_initialize_without_process_id_should_raise_error(client_server):
-    _, server = client_server
-    params = dictToObj({
-        "capabilities": {},
-        "rootUri": None
-    })
-    with pytest.raises(Exception):
-        server.lsp.bf_initialize(params)
-
-
-def test_initialize_without_root_uri_should_raise_error(client_server):
-    _, server = client_server
-    params = dictToObj({
-        "capabilities": {},
-        "processId": 1234,
-    })
-    with pytest.raises(Exception):
-        server.lsp.bf_initialize(params)
+    message = '\r\n'.join([
+        'Content-Length: ' + str(len(body)),
+        'Content-Type: application/vscode-jsonrpc; charset=utf-8',
+        '',
+        body,
+    ]).encode("utf-8")
+    future = server.lsp._server_request_futures["err"] = Future()
+    server.lsp.data_received(message)
+    with pytest.raises(JsonRpcException, match="message for you sir"):
+        future.result()
 
 
 def test_initialize_should_return_server_capabilities(client_server):
     _, server = client_server
-    params = dictToObj({
-        "capabilities": {},
-        "processId": 1234,
-        "rootUri": None
-    })
+    params = InitializeParams(
+        process_id=1234,
+        root_uri=Path(__file__).parent.as_uri(),
+        capabilities=ClientCapabilities(),
+    )
 
     server_capabilities = server.lsp.bf_initialize(params)
 
     assert isinstance(server_capabilities, InitializeResult)
 
 
-def test_response_object_fields():
-    # Result field set
-    response = JsonRPCResponseMessage(0, '2.0', 'result', None).without_none_fields()
-
-    assert hasattr(response, 'id')
-    assert hasattr(response, 'jsonrpc')
-    assert hasattr(response, 'result')
-    assert not hasattr(response, 'error')
-
-    # Error field set
-    response = JsonRPCResponseMessage(0, '2.0', None, 'error').without_none_fields()
-
-    assert hasattr(response, 'id')
-    assert hasattr(response, 'jsonrpc')
-    assert hasattr(response, 'error')
-    assert not hasattr(response, 'result')
-
-
 def test_to_lsp_name():
     f_name = 'text_document__did_open'
     name = 'textDocument/didOpen'
